/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：Java 信息安全性设计准则
 * 项目描述：用于检查Java源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.java.rules.checkers;


import org.sonar.check.Rule;
import org.sonar.java.ast.visitors.SubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;


@Rule(key = "DynamicCodeChecker")

//检测代码中包含动态代码执行操作时，工具进行提示
public class DynamicCodeChecker extends SubscriptionVisitor {

    @Override
    public List<Tree.Kind> nodesToVisit() {
        return Collections.singletonList(
                Tree.Kind.METHOD_INVOCATION
        );
    }

    @Override
    public void visitNode(Tree tree) {
        MethodInvocationTree node = (MethodInvocationTree) tree;
        System.out.println(node);
        var expressionTree = node.methodSelect();
        if (expressionTree instanceof MemberSelectExpressionTree) {
            var exprTree = (MemberSelectExpressionTree) expressionTree;
            var name = exprTree.identifier();
            if ("eval".equals(name.toString())) {
                var varNameNode = exprTree.expression();
                if (varNameNode instanceof IdentifierTree) {
                    var varName = (IdentifierTree) varNameNode;
                    var symbol = varName.symbol();
                    var varDecler = symbol.declaration();
                    if (varDecler != null) {
                        var variableTree = (VariableTree) varDecler;
                        var typeName = variableTree.type().toString();
                        if ("ScriptEngine".equals(typeName)) {
                            context.reportIssue(this, tree, "程序设计时禁止动态构建代码进行功能实现");
                        }
                    }
                }
            }
        }
    }

}
